@@ -13,7 +13,7 @@
  * You should have received a copy of the GNU General Public License
  * along with the hkl library.  If not, see <http://www.gnu.org/licenses/>.
  *
- * Copyright (C) 2003-2024 Synchrotron SOLEIL
+ * Copyright (C) 2003-2025 Synchrotron SOLEIL
  *                         L'Orme des Merisiers Saint-Aubin
  *                         BP 48 91192 GIF-sur-YVETTE CEDEX
  *
@@ -42,6 +42,7 @@ typedef enum _HklBinocularsDetectorEnum
         HKL_BINOCULARS_DETECTOR_MERLIN_MEDIPIX_3RX_QUAD,
         HKL_BINOCULARS_DETECTOR_MERLIN_MEDIPIX_3RX_QUAD_512,
         HKL_BINOCULARS_DETECTOR_CIRPAD,
+	HKL_BINOCULARS_DETECTOR_RIGAKU_XSPA_1M,
         /* Add new your detectors here */
         HKL_BINOCULARS_DETECTOR_NUM_DETECTORS,
         /* wip */
@@ -76,6 +77,11 @@ HKLAPI extern void hkl_binoculars_detector_2d_sixs_calibration(HklBinocularsDete
 HKLAPI extern uint32_t *hkl_binoculars_detector_2d_fake_image_uint32(HklBinocularsDetectorEnum n,
                                                                      size_t *n_pixels);
 
+HKLAPI extern int hkl_binoculars_detector_2d_img_load_into(HklBinocularsDetectorEnum n,
+                                                           const char *filename,
+                                                           void *buffer,
+                                                           int n_buffer);
+
 /***********/
 /* Limits  */
 /***********/
@@ -168,6 +174,7 @@ HKLAPI extern void hkl_binoculars_cube_fprintf(FILE *f, const HklBinocularsCube
 /* subprojection for QCustom */
 /* q_scannumber / tth_scannumber  ( scannumber = numero du scan) (optional) */
 /* q_timescan0 / tth_timescan0   (timescan0 = timestamp du 1er point du scan (constant pour 1 scan)) */
+/* q_chi_phi (q, chi = arccos(qz/q), phi = arctan2(qy,qx)) */
 
 /* Adding the polarization correction from fit2D */
 /* dans un premier temps faire (1 - (epsilon (0, 1, 0) . kf) ^ 2) */
@@ -191,6 +198,7 @@ HKLAPI extern void hkl_binoculars_cube_fprintf(FILE *f, const HklBinocularsCube
                                                       size_t n_limits,	\
                                                       const char *sample_axis)
 
+HKLAPI extern HKL_BINOCULARS_SPACE_ANGLES_DECL(double);
 HKLAPI extern HKL_BINOCULARS_SPACE_ANGLES_DECL(int32_t);
 HKLAPI extern HKL_BINOCULARS_SPACE_ANGLES_DECL(uint16_t);
 HKLAPI extern HKL_BINOCULARS_SPACE_ANGLES_DECL(uint32_t);
@@ -204,9 +212,9 @@ typedef enum _HklBinocularsQCustomSubProjectionEnum
         HKL_BINOCULARS_QCUSTOM_SUB_PROJECTION_Q_TIMESTAMP,
         HKL_BINOCULARS_QCUSTOM_SUB_PROJECTION_QPAR_QPER_TIMESTAMP,
         HKL_BINOCULARS_QCUSTOM_SUB_PROJECTION_QPAR_QPER,
-        HKL_BINOCULARS_QCUSTOM_SUB_PROJECTION_Q_PHI_QX,
-        HKL_BINOCULARS_QCUSTOM_SUB_PROJECTION_Q_PHI_QY,
-        HKL_BINOCULARS_QCUSTOM_SUB_PROJECTION_Q_PHI_QZ,
+        HKL_BINOCULARS_QCUSTOM_SUB_PROJECTION_Q_PHIX_QX,
+        HKL_BINOCULARS_QCUSTOM_SUB_PROJECTION_Q_PHIY_QY,
+        HKL_BINOCULARS_QCUSTOM_SUB_PROJECTION_Q_PHIZ_QZ,
         HKL_BINOCULARS_QCUSTOM_SUB_PROJECTION_Q_STEREO,
         HKL_BINOCULARS_QCUSTOM_SUB_PROJECTION_DELTALAB_GAMMALAB_SAMPLEAXIS,
         HKL_BINOCULARS_QCUSTOM_SUB_PROJECTION_X_Y_Z,
@@ -223,6 +231,9 @@ typedef enum _HklBinocularsQCustomSubProjectionEnum
         HKL_BINOCULARS_QCUSTOM_SUB_PROJECTION_Q_TIMESCAN0,
         HKL_BINOCULARS_QCUSTOM_SUB_PROJECTION_Q_SCANNUMBER,
         HKL_BINOCULARS_QCUSTOM_SUB_PROJECTION_TTH_SCANNUMBER,
+        HKL_BINOCULARS_QCUSTOM_SUB_PROJECTION_PHIX_Q_THETAX,
+        HKL_BINOCULARS_QCUSTOM_SUB_PROJECTION_PHIY_Q_THETAY,
+        HKL_BINOCULARS_QCUSTOM_SUB_PROJECTION_PHIZ_Q_THETAZ,
         /* Add new your subprojection in the same order than the haskell order here */
         HKL_BINOCULARS_QCUSTOM_NUM_SUBPROJECTIONS,
 } HklBinocularsQCustomSubProjectionEnum;
@@ -254,6 +265,7 @@ typedef enum _HklBinocularsQCustomSubProjectionEnum
                 )
 
 
+HKLAPI extern HKL_BINOCULARS_SPACE_QCUSTOM_DECL(double);
 HKLAPI extern HKL_BINOCULARS_SPACE_QCUSTOM_DECL(int32_t);
 HKLAPI extern HKL_BINOCULARS_SPACE_QCUSTOM_DECL(uint16_t);
 HKLAPI extern HKL_BINOCULARS_SPACE_QCUSTOM_DECL(uint32_t);
@@ -278,6 +290,7 @@ HKLAPI extern HKL_BINOCULARS_SPACE_QCUSTOM_DECL(uint32_t);
                                                    int do_polarisation_correction \
                 )
 
+HKLAPI extern HKL_BINOCULARS_SPACE_HKL_DECL(double);
 HKLAPI extern HKL_BINOCULARS_SPACE_HKL_DECL(int32_t);
 HKLAPI extern HKL_BINOCULARS_SPACE_HKL_DECL(uint16_t);
 HKLAPI extern HKL_BINOCULARS_SPACE_HKL_DECL(uint32_t);
@@ -303,6 +316,7 @@ HKLAPI extern HKL_BINOCULARS_SPACE_HKL_DECL(uint32_t);
                 )
 
 
+HKLAPI extern HKL_BINOCULARS_SPACE_TEST_DECL(double);
 HKLAPI extern HKL_BINOCULARS_SPACE_TEST_DECL(int32_t);
 HKLAPI extern HKL_BINOCULARS_SPACE_TEST_DECL(uint16_t);
 HKLAPI extern HKL_BINOCULARS_SPACE_TEST_DECL(uint32_t);
