@@ -17,7 +17,7 @@
 {-# LANGUAGE UndecidableInstances       #-}
 
 {-
-    Copyright  : Copyright (C) 2014-, 20242024 Synchrotron SOLEIL
+    Copyright  : Copyright (C) 2014-2025 Synchrotron SOLEIL
                                          L'Orme des Merisiers Saint-Aubin
                                          BP 48 91192 GIF-sur-YVETTE CEDEX
     License    : GPL3+
@@ -32,7 +32,6 @@ module Hkl.Binoculars.Config
     , Attenuation(..)
     , BinocularsPreConfig(..)
     , Capabilities(..)
-    , ConfigContent(..)
     , ConfigRange(..)
     , Degree(..)
     , DestinationTmpl(..)
@@ -41,6 +40,7 @@ module Hkl.Binoculars.Config
     , HasFieldComment(..)
     , HasFieldValue(..)
     , HasIniConfig(..)
+    , HasIniParser(..)
     , InputRange(..)
     , InputTmpl(..)
     , InputType(..)
@@ -55,6 +55,7 @@ module Hkl.Binoculars.Config
     , SampleAxis(..)
     , auto
     , auto'
+    , autoJSON
     , destination'
     , elemFDef
     , elemFDef'
@@ -63,6 +64,7 @@ module Hkl.Binoculars.Config
     , eitherF
     , files
     , getCapabilities
+    , getInitialScannumber
     , getMask
     , getPreConfig
     , mergeIni
@@ -80,26 +82,32 @@ import           Control.Monad.Catch               (MonadCatch, MonadThrow,
                                                     throwM, try)
 import           Control.Monad.Catch.Pure          (runCatch)
 import           Control.Monad.IO.Class            (MonadIO)
-import           Data.Aeson                        (FromJSON (..), ToJSON (..))
+import           Data.Aeson                        (FromJSON (..), ToJSON (..),
+                                                    eitherDecodeStrict, encode)
 import           Data.Attoparsec.Text              (Parser, char, decimal,
                                                     double, parseOnly, peekChar,
                                                     satisfy, sepBy, signed,
-                                                    takeText, takeTill)
+                                                    skipSpace, takeText,
+                                                    takeTill, (<?>))
+import           Data.ByteString.Lazy              (toStrict)
+import           Data.Char                         (isSpace)
 import           Data.Either.Combinators           (maybeToRight)
 import           Data.Either.Extra                 (mapLeft, mapRight)
 import           Data.Foldable                     (foldl')
-import           Data.HashMap.Strict               (HashMap, unionWith)
 import           Data.Hashable                     (Hashable)
+import           Data.HashMap.Strict               (HashMap, unionWith)
 import           Data.Ini                          (Ini (..), printIni)
-import           Data.Ini.Config                   (fieldMbOf, parseIniFile,
-                                                    section)
+import           Data.Ini.Config                   (IniParser, fieldMbOf,
+                                                    fieldOf, parseIniFile,
+                                                    section, sectionMb)
 import           Data.Ini.Config.Bidir             (FieldValue (..), IniSpec,
                                                     bool, field, getIniValue,
                                                     ini, listWithSeparator,
                                                     number, parseIni, section,
                                                     text, (.=))
-import           Data.List                         (find, isInfixOf, length)
-import           Data.List.NonEmpty                (NonEmpty (..), map)
+import           Data.List                         (elemIndex, find, isInfixOf,
+                                                    length)
+import           Data.List.NonEmpty                (NonEmpty (..), head, map)
 import           Data.Maybe                        (catMaybes, fromMaybe)
 import           Data.Text                         (Text, breakOn, cons, drop,
                                                     empty, findIndex,
@@ -107,7 +115,9 @@ import           Data.Text                         (Text, breakOn, cons, drop,
                                                     length, lines, pack,
                                                     replace, singleton, strip,
                                                     take, takeWhile, toLower,
-                                                    unlines, unpack, unwords)
+                                                    unlines, unpack, unwords,
+                                                    words)
+import           Data.Text.Encoding                (decodeUtf8, encodeUtf8)
 import           Data.Text.IO                      (readFile)
 import           Data.Typeable                     (Proxy (..), Typeable,
                                                     typeRep)
@@ -138,6 +148,7 @@ import           Prelude                           hiding (drop, length, lines,
 import           Hkl.C.Binoculars
 import           Hkl.Detector
 import           Hkl.Exception
+import           Hkl.Geometry
 import           Hkl.Lattice
 import           Hkl.Repa
 import           Hkl.Types
@@ -162,6 +173,10 @@ import           Paths_hkl
 --  - delta_lab, gamma_lab, <sample-axis>
 --  - gamma_lab, delta_lab, <sample_axis>
 
+
+word :: Parser Text
+word = takeTill isSpace <?> "word"
+
 -- Class FieldEmitter
 
 class FieldEmitter a where
@@ -188,6 +203,13 @@ auto = fieldvalue
 auto' :: HasFieldValue a => Text -> Either String a
 auto' = fvParse fieldvalue
 
+autoJSON :: (FromJSON a, ToJSON a) => FieldValue a
+autoJSON =
+    FieldValue { fvParse = eitherDecodeStrict . encodeUtf8
+               , fvEmit = decodeUtf8 . toStrict . encode
+               }
+
+
 instance HasFieldValue Bool where
   fieldvalue = bool
 
@@ -247,6 +269,11 @@ pairWithSeparator' left sep right = FieldValue
 instance HasFieldValue (Int, Int) where
   fieldvalue = pairWithSeparator' number' "," number'
 
+-- Class HasIniParser
+
+class HasIniParser a where
+    iniParser :: IniParser a
+
 -- Class HasIniConfig
 
 readConfig :: Maybe FilePath -> IO ConfigContent
@@ -311,6 +338,20 @@ instance HasFieldValue Angstrom where
 newtype Attenuation = Attenuation { unAttenuation :: Double }
   deriving (Eq, Show)
 
+-- Axis
+
+instance FieldEmitter Axis where
+  fieldEmitter (Axis n t u) = fieldEmitter n <> fieldEmitter t <> fieldEmitter u
+
+instance FieldParsable Axis where
+  fieldParser = Axis
+                <$> fieldParser
+                <*> fieldParser
+                <*> fieldParser
+
+instance HasFieldValue Axis where
+  fieldvalue = parsable
+
 -- Capabilities
 
 data Capabilities = Capabilities Int Int
@@ -344,6 +385,11 @@ instance FieldParsable ConfigRange where
 instance HasFieldValue ConfigRange where
   fieldvalue = parsable
 
+getInitialScannumber :: ConfigRange -> Scannumber
+getInitialScannumber (ConfigRange ne) = let (InputRange int) = Data.List.NonEmpty.head ne
+                                            sn0 = inf int
+                                        in Scannumber sn0
+
 -- DestinationTmpl
 
 newtype DestinationTmpl =
@@ -359,6 +405,35 @@ instance HasFieldValue DestinationTmpl where
     , fvEmit = \(DestinationTmpl t) -> t
     }
 
+instance FieldEmitter Double where
+  fieldEmitter d = pack $ printf "%f" d
+
+-- Geometry
+
+instance HasIniParser (Maybe Geometry) where
+    iniParser
+        = do res <- sectionMb "geometry" $
+                   (,)
+                   <$> fieldOf "geometry_sample" (parseOnly words')
+                   <*> fieldMbOf "geometry_detector" (parseOnly words')
+             case res of
+               Nothing -> pure Nothing
+               Just (sample, mdetector) -> do
+                                         sample_axes <- mapM iniParser'Axis sample
+                                         detector_axes <- case mdetector of
+                                                           Nothing       -> pure []
+                                                           Just detector -> mapM iniParser'Axis detector
+                                         pure $ mk'Geometry sample_axes detector_axes
+        where
+          words' :: Parser [Text]
+          words' = Data.Text.words <$> takeText
+
+          iniParser'Axis :: Text -> IniParser Axis
+          iniParser'Axis n
+              = do (t, u) <- Data.Ini.Config.section "geometry" $ fieldOf ("axis_" <> n) (parseOnly ((,) <$> fieldParser <*> fieldParser))
+                   pure $ Axis (unpack n) t u
+
+
 -- HklBinocularsQCustomSubProjectionEnum
 
 instance FieldEmitter HklBinocularsQCustomSubProjectionEnum where
@@ -386,6 +461,9 @@ instance FieldEmitter HklBinocularsQCustomSubProjectionEnum where
   fieldEmitter HklBinocularsQCustomSubProjectionEnum'QTimescan0           = "q_timescan0"
   fieldEmitter HklBinocularsQCustomSubProjectionEnum'QScannumber          = "q_scannumber"
   fieldEmitter HklBinocularsQCustomSubProjectionEnum'TthScannumber        = "tth_scannumber"
+  fieldEmitter HklBinocularsQCustomSubProjectionEnum'PhixQThetax          = "phix_q_thetax"
+  fieldEmitter HklBinocularsQCustomSubProjectionEnum'PhiyQThetay          = "phiy_q_thetay"
+  fieldEmitter HklBinocularsQCustomSubProjectionEnum'PhizQThetaz          = "phiz_q_thetaz"
 
 instance FieldParsable HklBinocularsQCustomSubProjectionEnum where
   fieldParser = go . strip . uncomment . toLower =<< takeText
@@ -524,6 +602,7 @@ instance HasFieldValue InputTypeDeprecated where
   fieldvalue = parsable
 
 data InputType = CristalK6C
+               | Custom
                | DiffabsCirpad
                | MarsFlyscan
                | MarsSbs
@@ -546,6 +625,7 @@ instance Arbitrary InputType where
 
 instance FieldEmitter InputType where
   fieldEmitter CristalK6C        = "cristal:k6c"
+  fieldEmitter Custom            = "custom"
   fieldEmitter DiffabsCirpad     = "diffabs:cirpad"
   fieldEmitter MarsFlyscan       = "mars:flyscan"
   fieldEmitter MarsSbs           = "mars:sbs"
@@ -773,8 +853,8 @@ instance HasFieldValue (Resolutions DIM2) where
       parse t = do
         rs <- (fvParse $ listWithSeparator "," auto) t
         case Data.List.length rs of
-          1 -> Right (Resolutions2 (head rs) (head rs))
-          2 -> Right (Resolutions2 (head rs) (rs !! 1))
+          1 -> Right (Resolutions2 (Prelude.head rs) (Prelude.head rs))
+          2 -> Right (Resolutions2 (Prelude.head rs) (rs !! 1))
           _ -> Left "Need one or two resolutions values for this projection"
 
       emit :: Resolutions DIM2 -> Text
@@ -797,8 +877,8 @@ instance HasFieldValue (Resolutions DIM3) where
       parse t = do
         rs <- (fvParse $ listWithSeparator "," auto) t
         case Data.List.length rs of
-          1 -> Right (Resolutions3 (head rs) (head rs) (head rs))
-          3 -> Right (Resolutions3 (head rs) (rs !! 1) (rs !! 2))
+          1 -> Right (Resolutions3 (Prelude.head rs) (Prelude.head rs) (Prelude.head rs))
+          3 -> Right (Resolutions3 (Prelude.head rs) (rs !! 1) (rs !! 2))
           _ -> Left "Need one or three resolutions values for this projection"
 
       emit :: Resolutions DIM3 -> Text
@@ -925,6 +1005,56 @@ instance HasFieldValue SampleAxis where
 instance Arbitrary SampleAxis where
   arbitrary = pure $ SampleAxis "omega"
 
+-- String
+
+instance FieldEmitter String where
+  fieldEmitter = pack
+
+instance FieldParsable String where
+  fieldParser = unpack <$> takeText
+
+-- Transformation
+
+instance FieldEmitter Transformation where
+  fieldEmitter NoTransformation = "no-transformation"
+  fieldEmitter (Rotation x y z) = "rotation " <> fieldEmitter x <> " " <> fieldEmitter y <> " " <> fieldEmitter z
+  fieldEmitter (Translation x y z) = "translation " <> fieldEmitter x <> " " <> fieldEmitter y <> " " <> fieldEmitter z
+
+instance FieldParsable Transformation where
+    fieldParser = do
+      t <- skipSpace *> word
+      case t of
+        "no-transformation" -> pure NoTransformation
+        "rotation" -> Rotation
+                     <$> (skipSpace *> double)
+                     <*> (skipSpace *> double)
+                     <*> (skipSpace *> double)
+                     <?> "rotation maformed expect: 'rotation <x> <y> <z>'"
+        "translation" -> Translation
+                     <$> (skipSpace *> double)
+                     <*> (skipSpace *> double)
+                     <*> (skipSpace *> double)
+                     <?> "translation maformed expect: 'translation <x> <y> <z>'"
+        _ -> fail ("unknown transformation: " <> unpack t <> " only 'no-transformation', 'rotation', 'translation'")
+
+-- Unit
+
+instance FieldEmitter Unit where
+  fieldEmitter Unit'NoUnit            = "ua"
+  fieldEmitter Unit'Angle'Degree      = "degree"
+  fieldEmitter Unit'Length'MilliMeter = "millimeter"
+
+instance FieldParsable Unit where
+    fieldParser = do
+      t <- skipSpace *> word
+      let units = [minBound..maxBound] :: [Unit]
+      let supported = [ fieldEmitter u | u <- units ]
+      let mi = elemIndex t supported
+      case mi of
+        Just i -> pure $ units !! i
+        Nothing -> fail $ unpack ("unknown unit '" <> t <> "' only supported are "
+                                 <> unwords [ "\'" <> t' <> "\'" | t' <- supported ])
+
 -- BinocularsPreConfig
 
 newtype BinocularsPreConfig =
@@ -975,7 +1105,7 @@ isHdf5 :: Path Abs File -> Bool
 isHdf5 p =
   case (fileExtension p :: Maybe [Char]) of
     Nothing  -> False
-    Just ext -> ext `elem` [".nxs"]
+    Just ext -> ext == ".nxs"
 
 -- isInConfigRange :: Maybe InputTmpl -> Maybe ConfigRange -> FilePath -> Bool
 -- isInConfigRange mtmpl mr f
@@ -995,7 +1125,7 @@ isHdf5 p =
 
 matchInputRange' :: String -> Path Abs File -> [Int] -> [Maybe ScanFilePath]
 matchInputRange' tmpl f is =
-  case find (\i -> printf tmpl i `Data.List.isInfixOf` (toFilePath f)) is of
+  case find (\i -> printf tmpl i `Data.List.isInfixOf` toFilePath f) is of
     Nothing -> []
     Just i  -> [Just (ScanFilePath f (Scannumber i))]
 
@@ -1017,9 +1147,7 @@ files :: (MonadThrow m, MonadIO m)
        -> Maybe InputTmpl
        -> m [ScanFilePath]
 files md mr mt =
-  do dir <- case md of
-             Nothing -> getCurrentDir
-             Just d  -> pure d
+  do dir <- maybe getCurrentDir pure md
 
      fs <- walkDirAccum Nothing
           (\_root _dirs fs ->
@@ -1077,10 +1205,37 @@ replace' proj msub i l dtmpl midx = unpack
                           . unDestinationTmpl . addOverwrite midx $ dtmpl
 
 
+-- geometry_sample = pitch, mu
+-- geometry_detector = pitch, gamma, delta
+
+-- axis_pitch = rotation 0 -1 0 deg
+-- axis_mu = rotation 0 0 1 deg
+-- axis_gamma = rotation 0 0 1 deg
+-- axis_delta = rotation 0 -1 0 deg
+
+
+-- parse'Axis :: Text -> IniParser Axis
+-- parse'Axis n
+--   = Data.Ini.Config.section "geometry" $ do
+--   fieldOf ("axis_" <> n) auto'
+
+-- parse'Geometry :: IniParser (Maybe Geometry)
+-- parse'Geometry
+--   = do
+--   msamples <-  Data.Ini.Config.section "geometry" $ fieldOf "geometry_sample" auto'
+--   mdetectors <-  Data.Ini.Config.section "geometry" $ fieldOf "geometry_detector" auto'
+--   axes_samples <- mapM (\a -> parse'Axis a) msamples
+--   axes_detectors <- mapM (\a -> parse'Axis a) mdetectors
+--   let mg = mk'Geometry axes_samples axes_detectors
+--   pure $ mk'Geometry axes_samples axes_detectors
+
 ------------------
 -- Parser utils --
 ------------------
 
+parseError :: Text -> Text -> String -> Either String r
+parseError s f err = Left ("Can not parse " <> unpack s <> "." <> unpack f <> ": " <> err)
+
 parse' :: HasFieldValue b => Text -> Text -> Text -> Either String (Maybe b)
 parse' c s f = parseIniFile c $ Data.Ini.Config.section s (fieldMbOf f auto')
 
@@ -1090,7 +1245,7 @@ eitherF _ (Right b) fb = fb b
 
 parseF :: HasFieldValue b
        => Text -> Text -> Text -> (Maybe b -> Either String r) -> Either String r
-parseF c s f = eitherF error (parse' c s f)
+parseF c s f = eitherF (parseError s f) (parse' c s f)
 
 parseFDef :: HasFieldValue p
           => Text -> Text -> Text -> p -> Either String p
@@ -1100,12 +1255,16 @@ parseFDef c s f def = parseF c s f $ \case
 
 parseMb ::  HasFieldValue r
         => Text -> Text -> Text -> Either String (Maybe r)
-parseMb c s f = eitherF error (parse' c s f) pure
+parseMb c s f = eitherF (parseError s f) (parse' c s f) pure
 
 parseMbDef :: HasFieldValue r
            => Text -> Text -> Text -> Maybe r -> Maybe r
 parseMbDef c s f def = eitherF error (parse' c s f) (<|> def)
 
+-----------------
+-- Ini Emitter --
+-----------------
+
 elemFDef' :: HasFieldComment w => Text -> (v -> w) -> v -> v -> [(Text, Text)]
 elemFDef' k f v d = elemFDef k f v d (fieldComment (f v))
 
