@@ -1,8 +1,8 @@
 {-# LANGUAGE DataKinds             #-}
-{-# LANGUAGE DeriveAnyClass        #-}
 {-# LANGUAGE DeriveGeneric         #-}
 {-# LANGUAGE FlexibleContexts      #-}
 {-# LANGUAGE FlexibleInstances     #-}
+{-# LANGUAGE LambdaCase            #-}
 {-# LANGUAGE MultiParamTypeClasses #-}
 {-# LANGUAGE OverloadedStrings     #-}
 {-# LANGUAGE RecordWildCards       #-}
@@ -13,7 +13,7 @@
 {-# OPTIONS_GHC -fno-warn-orphans #-}
 
 {-
-    Copyright  : Copyright (C) 2014-2024 Synchrotron SOLEIL
+    Copyright  : Copyright (C) 2014-2025 Synchrotron SOLEIL
                                          L'Orme des Merisiers Saint-Aubin
                                          BP 48 91192 GIF-sur-YVETTE CEDEX
     License    : GPL3+
@@ -25,7 +25,7 @@
 
 module Hkl.Binoculars.Projections.Hkl
     ( Config(..)
-    , DataFrameHkl'(..)
+    , DataFrameHkl(..)
     , newHkl
     , processHkl
     , updateHkl
@@ -39,15 +39,11 @@ import           Control.Monad.IO.Class             (MonadIO (liftIO))
 import           Control.Monad.Logger               (MonadLogger, logDebugN,
                                                      logInfoN)
 import           Control.Monad.Reader               (MonadReader, ask)
-import           Data.Aeson                         (FromJSON, ToJSON,
-                                                     eitherDecode', encode)
-import           Data.ByteString.Lazy               (fromStrict, toStrict)
-import           Data.Functor.Identity              (Identity)
+import           Data.Aeson                         (FromJSON, ToJSON)
 import           Data.HashMap.Strict                (fromList)
 import           Data.Ini                           (Ini (..))
-import           Data.Ini.Config.Bidir              (FieldValue (..))
+import           Data.Maybe                         (fromMaybe)
 import           Data.Text                          (pack, unpack)
-import           Data.Text.Encoding                 (decodeUtf8, encodeUtf8)
 import           Data.Text.IO                       (putStr)
 import           Data.Vector.Storable.Mutable       (unsafeWith)
 import           Foreign.C.Types                    (CDouble (..))
@@ -55,7 +51,7 @@ import           Foreign.ForeignPtr                 (withForeignPtr)
 import           GHC.Generics                       (Generic)
 import           Path                               (Abs, Dir, Path)
 import           Pipes                              (await, each, runEffect,
-                                                     (>->))
+                                                     yield, (>->))
 import           Pipes.Prelude                      (filter, map, tee, toListM)
 import           Pipes.Safe                         (runSafeP, runSafeT)
 import           Text.Printf                        (printf)
@@ -70,34 +66,49 @@ import           Hkl.C.Binoculars
 import           Hkl.DataSource
 import           Hkl.Detector
 import           Hkl.Geometry
-import           Hkl.HKD
 import           Hkl.Image
 import           Hkl.Orphan                         ()
 import           Hkl.Repa
 import           Hkl.Sample
+import           Hkl.Types
 import           Hkl.Utils
 
 ----------------
 -- DataPath's --
 ----------------
 
-data DataFrameHkl' f
-  = DataFrameHkl
-    { dataFrameHkl'DataFrameQCustom :: HKD f DataFrameQCustom
-    , dataFrameHkl'Sample           :: HKD f Sample
-    }
+data DataFrameHkl
+    = DataFrameHkl
+      DataFrameQCustom
+      Sample
+
+data DSDataFrameHkl (k :: DSKind)
+  = DSDataFrameHkl
+    (DSWrap_ DSDataFrameQCustom k)
+    (DSWrap_ DSSample k)
   deriving (Generic)
 
-deriving instance Show (DataFrameHkl' DataSourcePath)
-instance FromJSON (DataFrameHkl' DataSourcePath)
-instance ToJSON (DataFrameHkl' DataSourcePath)
 
-defaultDataSourcePath'DataFrameHkl :: DataFrameHkl' DataSourcePath
-defaultDataSourcePath'DataFrameHkl = DataFrameHkl
-                                     default'DataSourcePath'DataFrameQCustom
-                                     default'DataSourcePath'Sample
+deriving instance Show (DSDataFrameHkl DSPath)
+instance FromJSON (DSDataFrameHkl DSPath)
+instance ToJSON (DSDataFrameHkl DSPath)
+instance DataSource DSDataFrameHkl
 
-instance HasFieldComment (DataFrameHkl' DataSourcePath) where
+instance Is1DStreamable (DSDataFrameHkl DSAcq) DataFrameHkl where
+    extract1DStreamValue (DSDataFrameHkl q s) i
+        = DataFrameHkl
+          <$> extract1DStreamValue q i
+          <*> extract0DStreamValue s
+
+defaultDataSource'DataFrameHkl :: DSWrap_ DSDataFrameHkl DSPath
+defaultDataSource'DataFrameHkl
+    = [ DSDataFrameHkl
+        default'DataSource'DataFrameQCustom
+        default'DataSource'Sample
+      ]
+
+
+instance HasFieldComment [DSDataFrameHkl DSPath] where
   fieldComment _ = [ "`datapath` internal value used to find the data in the data file."
                    , ""
                    , "This value is for expert only."
@@ -105,11 +116,8 @@ instance HasFieldComment (DataFrameHkl' DataSourcePath) where
                    , "default value: <not set>"
                    ]
 
-instance HasFieldValue (DataFrameHkl' DataSourcePath) where
-  fieldvalue = FieldValue
-               { fvParse = eitherDecode' . fromStrict . encodeUtf8
-               , fvEmit = decodeUtf8 . toStrict . encode
-               }
+instance HasFieldValue [DSDataFrameHkl DSPath] where
+  fieldvalue = autoJSON
 
 ------------
 -- Config --
@@ -117,13 +125,15 @@ instance HasFieldValue (DataFrameHkl' DataSourcePath) where
 
 overload'DataSourcePath'DataFrameHkl :: Config Common
                                      -> Config Sample
-                                     -> DataFrameHkl' DataSourcePath
-                                     -> DataFrameHkl' DataSourcePath
-overload'DataSourcePath'DataFrameHkl common sample (DataFrameHkl qCustomPath samplePath)
-  = DataFrameHkl newQCustomPath newSamplePath
-  where
-    newQCustomPath = overload'DataSourcePath'DataFrameQCustom common Nothing qCustomPath
-    newSamplePath = overload'DataSourcePath'Sample sample samplePath
+                                     -> DSWrap_ DSDataFrameHkl DSPath
+                                     -> DSWrap_ DSDataFrameHkl DSPath
+overload'DataSourcePath'DataFrameHkl common sample
+    = Prelude.map
+      (\(DSDataFrameHkl qCustomPath samplePath) ->
+           let newQCustomPath = overload'DataSource'DataFrameQCustom common Nothing qCustomPath
+               newSamplePath = overload'DataSource'Sample sample samplePath
+           in DSDataFrameHkl newQCustomPath newSamplePath
+      )
 
 instance HasIniConfig 'HklProjection where
   data Config 'HklProjection
@@ -133,7 +143,7 @@ instance HasIniConfig 'HklProjection where
       , binocularsConfig'Hkl'ProjectionType         :: ProjectionType
       , binocularsConfig'Hkl'ProjectionResolution   :: Resolutions DIM3
       , binocularsConfig'Hkl'ProjectionLimits       :: Maybe (RLimits DIM3)
-      , binocularsConfig'Hkl'DataPath               :: DataFrameHkl' DataSourcePath
+      , binocularsConfig'Hkl'DataPath               :: DSWrap_ DSDataFrameHkl DSPath
       } deriving (Generic)
 
   newtype Args 'HklProjection
@@ -146,7 +156,7 @@ instance HasIniConfig 'HklProjection where
         , binocularsConfig'Hkl'ProjectionType = HklProjection
         , binocularsConfig'Hkl'ProjectionResolution = Resolutions3 0.01 0.01 0.01
         , binocularsConfig'Hkl'ProjectionLimits  = Nothing
-        , binocularsConfig'Hkl'DataPath = defaultDataSourcePath'DataFrameHkl
+        , binocularsConfig'Hkl'DataPath = defaultDataSource'DataFrameHkl
         }
 
   getConfig content@(ConfigContent cfg) (Args'HklProjection mr) capabilities
@@ -155,10 +165,10 @@ instance HasIniConfig 'HklProjection where
            binocularsConfig'Hkl'ProjectionType <- parseFDef cfg "projection" "type" (binocularsConfig'Hkl'ProjectionType defaultConfig)
            binocularsConfig'Hkl'ProjectionResolution <- parseFDef cfg "projection" "resolution" (binocularsConfig'Hkl'ProjectionResolution defaultConfig)
            binocularsConfig'Hkl'ProjectionLimits <- parseMb cfg "projection" "limits"
-           binocularsConfig'Hkl'DataPath <- pure $ eitherF (const $ guess'DataSourcePath'DataFrameHkl binocularsConfig'Hkl'Common binocularsConfig'Hkl'Sample content) (parse' cfg "input" "datapath")
-                                           (\md -> case md of
-                                                    Nothing -> guess'DataSourcePath'DataFrameHkl binocularsConfig'Hkl'Common binocularsConfig'Hkl'Sample content
-                                                    Just d  ->  overload'DataSourcePath'DataFrameHkl binocularsConfig'Hkl'Common binocularsConfig'Hkl'Sample d)
+           let binocularsConfig'Hkl'DataPath = eitherF (const $ guess'DataSourcePath'DataFrameHkl binocularsConfig'Hkl'Common binocularsConfig'Hkl'Sample content) (parse' cfg "input" "datapath")
+                                               (\case
+                                                 Nothing -> guess'DataSourcePath'DataFrameHkl binocularsConfig'Hkl'Common binocularsConfig'Hkl'Sample content
+                                                 Just d  ->  overload'DataSourcePath'DataFrameHkl binocularsConfig'Hkl'Common binocularsConfig'Hkl'Sample d)
            pure BinocularsConfig'Hkl{..}
 
   toIni c = toIni (binocularsConfig'Hkl'Common c)
@@ -180,7 +190,7 @@ instance HasIniConfig 'HklProjection where
 ----------------
 
 {-# INLINE spaceHkl #-}
-spaceHkl :: Detector b DIM2 -> Array F DIM3 Double -> Resolutions DIM3 -> Maybe (RLimits DIM3) -> Bool -> Space DIM3 -> DataFrameHkl' Identity -> IO (DataFrameSpace DIM3)
+spaceHkl :: Detector b DIM2 -> Array F DIM3 Double -> Resolutions DIM3 -> Maybe (RLimits DIM3) -> Bool -> Space DIM3 -> DataFrameHkl -> IO (DataFrameSpace DIM3)
 spaceHkl det pixels rs mlimits doPolarizationCorrection space@(Space fSpace) (DataFrameHkl (DataFrameQCustom att g img mmask _ _ _) samplePath) = do
   withNPixels det $ \nPixels ->
     withGeometry g $ \geometry ->
@@ -192,6 +202,8 @@ spaceHkl det pixels rs mlimits doPolarizationCorrection space@(Space fSpace) (Da
     withMaybeLimits mlimits rs $ \nlimits limits ->
     withForeignPtr fSpace $ \pSpace -> do
     case img of
+      (ImageDouble arr) -> unsafeWith arr $ \i -> do
+        {-# SCC "hkl_binoculars_space_hkl_double" #-} c'hkl_binoculars_space_hkl_double pSpace geometry sample i nPixels (CDouble . unAttenuation $ att) pix (toEnum ndim) dims r (toEnum nr) mask'' limits (toEnum nlimits) (toEnum . fromEnum $ doPolarizationCorrection)
       (ImageInt32 arr) -> unsafeWith arr $ \i -> do
         {-# SCC "hkl_binoculars_space_hkl_int32_t" #-} c'hkl_binoculars_space_hkl_int32_t pSpace geometry sample i nPixels (CDouble . unAttenuation $ att) pix (toEnum ndim) dims r (toEnum nr) mask'' limits (toEnum nlimits) (toEnum . fromEnum $ doPolarizationCorrection)
       (ImageWord16 arr) -> unsafeWith arr $ \i -> do
@@ -281,19 +293,21 @@ processHklP = do
 
 -- FramesHklP
 
-instance ChunkP (DataFrameHkl' DataSourcePath) where
-  chunkP sf sl (DataFrameHkl p _) = chunkP sf sl p
+instance ChunkP [DSDataFrameHkl DSPath] where
+    chunkP mSkipFirst mSkipLast ps =
+      skipMalformed $ forever $ do
+      sfp <- await
+      withScanFileP sfp $ \f' ->
+        withDataSourcesP f' ps $ \_p p' -> do
+          (DataSourceShape'Range (Z :. f) (Z :. t)) <- ds'Shape p'
+          yield $ cclip (fromMaybe 0 mSkipFirst) (fromMaybe 0 mSkipLast) (Chunk sfp f (t - 1))
 
-instance FramesP (DataFrameHkl' DataSourcePath) (DataFrameHkl' Identity) where
-  framesP (DataFrameHkl qcustom sample) = skipMalformed $ forever $ do
+instance FramesP [DSDataFrameHkl DSPath] DataFrameHkl where
+  framesP ps = skipMalformed $ forever $ do
     (fp, js) <- await
     withScanFileP fp $ \f ->
-      withDataSourceP f qcustom $ \qcustomAcq ->
-      withDataSourceP f sample $ \sampleAcq ->
-      forM_ js (\j -> tryYield ( DataFrameHkl
-                                <$> extract1DStreamValue qcustomAcq j
-                                <*> extract0DStreamValue sampleAcq
-                              ))
+      withDataSourcesP f ps $ \_p p' ->
+      forM_ js (tryYield .  extract1DStreamValue p')
 
 ------------
 -- Inputs --
@@ -302,11 +316,12 @@ instance FramesP (DataFrameHkl' DataSourcePath) (DataFrameHkl' Identity) where
 guess'DataSourcePath'DataFrameHkl :: Config Common
                                   -> Config Sample
                                   -> ConfigContent
-                                  -> DataFrameHkl' DataSourcePath
+                                  -> DSWrap_ DSDataFrameHkl DSPath
 guess'DataSourcePath'DataFrameHkl common sample content
-  = DataFrameHkl
-    (guess'DataSourcePath'DataFrameQCustom common Nothing content)
-    (guess'DataSourcePath'Sample common sample)
+  = [ DSDataFrameHkl
+      (guess'DataSource'DataFrameQCustom common Nothing content)
+      (guess'DataSource'Sample common sample)
+    ]
 
 ---------
 -- Cmd --
