@@ -1,8 +1,8 @@
 {-# LANGUAGE DataKinds             #-}
-{-# LANGUAGE DeriveAnyClass        #-}
 {-# LANGUAGE DeriveGeneric         #-}
 {-# LANGUAGE FlexibleContexts      #-}
 {-# LANGUAGE FlexibleInstances     #-}
+{-# LANGUAGE LambdaCase            #-}
 {-# LANGUAGE MultiParamTypeClasses #-}
 {-# LANGUAGE OverloadedStrings     #-}
 {-# LANGUAGE RecordWildCards       #-}
@@ -13,7 +13,7 @@
 {-# OPTIONS_GHC -fno-warn-orphans #-}
 
 {-
-    Copyright  : Copyright (C) 2014-2024 Synchrotron SOLEIL
+    Copyright  : Copyright (C) 2014-2025 Synchrotron SOLEIL
                                          L'Orme des Merisiers Saint-Aubin
                                          BP 48 91192 GIF-sur-YVETTE CEDEX
     License    : GPL3+
@@ -36,15 +36,11 @@ import           Control.Monad.IO.Class             (MonadIO (liftIO))
 import           Control.Monad.Logger               (MonadLogger, logDebugN,
                                                      logInfoN)
 import           Control.Monad.Reader               (MonadReader, ask)
-import           Data.Aeson                         (FromJSON, ToJSON,
-                                                     eitherDecode', encode)
-import           Data.ByteString.Lazy               (fromStrict, toStrict)
-import           Data.Functor.Identity              (Identity)
+import           Data.Aeson                         (FromJSON, ToJSON)
 import           Data.HashMap.Strict                (fromList)
 import           Data.Ini                           (Ini (..))
-import           Data.Ini.Config.Bidir              (FieldValue (..))
+import           Data.Maybe                         (fromMaybe)
 import           Data.Text                          (pack, unpack)
-import           Data.Text.Encoding                 (decodeUtf8, encodeUtf8)
 import           Data.Text.IO                       (putStr)
 import           Data.Vector.Storable.Mutable       (unsafeWith)
 import           Foreign.C.Types                    (CDouble (..))
@@ -52,7 +48,7 @@ import           Foreign.ForeignPtr                 (withForeignPtr)
 import           GHC.Generics                       (Generic)
 import           Path                               (Abs, Dir, Path)
 import           Pipes                              (await, each, runEffect,
-                                                     (>->))
+                                                     yield, (>->))
 import           Pipes.Prelude                      (filter, map, tee, toListM)
 import           Pipes.Safe                         (runSafeP, runSafeT)
 import           Text.Printf                        (printf)
@@ -67,34 +63,48 @@ import           Hkl.C.Binoculars
 import           Hkl.DataSource
 import           Hkl.Detector
 import           Hkl.Geometry
-import           Hkl.HKD
 import           Hkl.Image
 import           Hkl.Orphan                         ()
 import           Hkl.Repa
 import           Hkl.Sample
+import           Hkl.Types
 import           Hkl.Utils
 
 ----------------
 -- DataPath's --
 ----------------
 
-data DataFrameTest' f
-  = DataFrameTest
-    { dataFrameTest'DataFrameQCustom :: HKD f DataFrameQCustom
-    , dataFrameTest'Sample           :: HKD f Sample
+data DataFrameTest
+    = DataFrameTest
+      DataFrameQCustom
+      Sample
+
+data DSDataFrameTest (k :: DSKind)
+  = DSDataFrameTest
+    { dataFrameTest'DataFrameQCustom :: DSWrap_ DSDataFrameQCustom k
+    , dataFrameTest'Sample           :: DSWrap_ DSSample k
     }
   deriving (Generic)
 
-deriving instance Show (DataFrameTest' DataSourcePath)
-instance FromJSON (DataFrameTest' DataSourcePath)
-instance ToJSON (DataFrameTest' DataSourcePath)
+deriving instance Show (DSDataFrameTest DSPath)
+instance FromJSON (DSDataFrameTest DSPath)
+instance ToJSON (DSDataFrameTest DSPath)
+instance DataSource DSDataFrameTest
 
-defaultDataSourcePath'DataFrameTest :: DataFrameTest' DataSourcePath
-defaultDataSourcePath'DataFrameTest = DataFrameTest
-                                     default'DataSourcePath'DataFrameQCustom
-                                     default'DataSourcePath'Sample
+instance Is1DStreamable (DSDataFrameTest DSAcq) DataFrameTest where
+    extract1DStreamValue (DSDataFrameTest q s) i =
+      DataFrameTest
+      <$> extract1DStreamValue q i
+      <*> extract0DStreamValue s
 
-instance HasFieldComment (DataFrameTest' DataSourcePath) where
+defaultDataSource'DataFrameTest :: DSWrap_ DSDataFrameTest DSPath
+defaultDataSource'DataFrameTest
+    = [ DSDataFrameTest
+        default'DataSource'DataFrameQCustom
+        default'DataSource'Sample
+      ]
+
+instance HasFieldComment [DSDataFrameTest DSPath] where
   fieldComment _ = [ "`datapath` internal value used to find the data in the data file."
                    , ""
                    , "This value is for expert only."
@@ -102,11 +112,8 @@ instance HasFieldComment (DataFrameTest' DataSourcePath) where
                    , "default value: <not set>"
                    ]
 
-instance HasFieldValue (DataFrameTest' DataSourcePath) where
-  fieldvalue = FieldValue
-               { fvParse = eitherDecode' . fromStrict . encodeUtf8
-               , fvEmit = decodeUtf8 . toStrict . encode
-               }
+instance HasFieldValue [DSDataFrameTest DSPath] where
+  fieldvalue = autoJSON
 
 ------------
 -- Config --
@@ -114,13 +121,15 @@ instance HasFieldValue (DataFrameTest' DataSourcePath) where
 
 overload'DataSourcePath'DataFrameTest :: Config Common
                                       -> Config Sample
-                                      -> DataFrameTest' DataSourcePath
-                                      -> DataFrameTest' DataSourcePath
-overload'DataSourcePath'DataFrameTest common sample (DataFrameTest qCustomPath samplePath)
-  = DataFrameTest newQCustomPath newSamplePath
-  where
-    newQCustomPath = overload'DataSourcePath'DataFrameQCustom common Nothing qCustomPath
-    newSamplePath = overload'DataSourcePath'Sample sample samplePath
+                                      -> DSWrap_ DSDataFrameTest DSPath
+                                      -> DSWrap_ DSDataFrameTest DSPath
+overload'DataSourcePath'DataFrameTest common sample
+    = Prelude.map
+      ( \(DSDataFrameTest qCustomPath samplePath) ->
+            let newQCustomPath = overload'DataSource'DataFrameQCustom common Nothing qCustomPath
+                newSamplePath = overload'DataSource'Sample sample samplePath
+            in DSDataFrameTest newQCustomPath newSamplePath
+      )
 
 instance HasIniConfig 'TestProjection where
   data Config 'TestProjection
@@ -130,7 +139,7 @@ instance HasIniConfig 'TestProjection where
       , binocularsConfig'Test'ProjectionType         :: ProjectionType
       , binocularsConfig'Test'ProjectionResolution   :: Resolutions DIM3
       , binocularsConfig'Test'ProjectionLimits       :: Maybe (RLimits DIM3)
-      , binocularsConfig'Test'DataPath               :: DataFrameTest' DataSourcePath
+      , binocularsConfig'Test'DataPath               :: DSWrap_ DSDataFrameTest DSPath
       } deriving (Generic)
 
   newtype Args 'TestProjection = Args'TestProjection (Maybe ConfigRange)
@@ -142,7 +151,7 @@ instance HasIniConfig 'TestProjection where
         , binocularsConfig'Test'ProjectionType = TestProjection
         , binocularsConfig'Test'ProjectionResolution = Resolutions3 0.01 0.01 0.01
         , binocularsConfig'Test'ProjectionLimits  = Nothing
-        , binocularsConfig'Test'DataPath = defaultDataSourcePath'DataFrameTest
+        , binocularsConfig'Test'DataPath = defaultDataSource'DataFrameTest
         }
 
   getConfig content@(ConfigContent cfg) (Args'TestProjection mr) capabilities
@@ -151,10 +160,10 @@ instance HasIniConfig 'TestProjection where
            binocularsConfig'Test'ProjectionType <- parseFDef cfg "projection" "type" (binocularsConfig'Test'ProjectionType defaultConfig)
            binocularsConfig'Test'ProjectionResolution <- parseFDef cfg "projection" "resolution" (binocularsConfig'Test'ProjectionResolution defaultConfig)
            binocularsConfig'Test'ProjectionLimits <- parseMb cfg "projection" "limits"
-           binocularsConfig'Test'DataPath <- (pure $ eitherF (const $ guess'DataSourcePath'DataFrameTest binocularsConfig'Test'Common binocularsConfig'Test'Sample content) (parse' cfg "input" "datapath")
-                                             (\md -> case md of
-                                                      Nothing -> guess'DataSourcePath'DataFrameTest binocularsConfig'Test'Common binocularsConfig'Test'Sample content
-                                                      Just d  ->  overload'DataSourcePath'DataFrameTest binocularsConfig'Test'Common binocularsConfig'Test'Sample d))
+           let binocularsConfig'Test'DataPath = eitherF (const $ guess'DataSourcePath'DataFrameTest binocularsConfig'Test'Common binocularsConfig'Test'Sample content) (parse' cfg "input" "datapath")
+                                                (\case
+                                                  Nothing -> guess'DataSourcePath'DataFrameTest binocularsConfig'Test'Common binocularsConfig'Test'Sample content
+                                                  Just d  ->  overload'DataSourcePath'DataFrameTest binocularsConfig'Test'Common binocularsConfig'Test'Sample d)
            pure BinocularsConfig'Test{..}
 
   toIni c = toIni (binocularsConfig'Test'Common c)
@@ -176,7 +185,7 @@ instance HasIniConfig 'TestProjection where
 ----------------
 
 {-# INLINE spaceTest #-}
-spaceTest :: Detector b DIM2 -> Array F DIM3 Double -> Resolutions DIM3 -> Maybe (RLimits DIM3) -> Bool -> Space DIM3 -> DataFrameTest' Identity -> IO (DataFrameSpace DIM3)
+spaceTest :: Detector b DIM2 -> Array F DIM3 Double -> Resolutions DIM3 -> Maybe (RLimits DIM3) -> Bool -> Space DIM3 -> DataFrameTest -> IO (DataFrameSpace DIM3)
 spaceTest det pixels rs mlimits doPolarizationCorrection space@(Space fSpace) (DataFrameTest (DataFrameQCustom att g img mmask _ _ _) samplePath) = do
   withNPixels det $ \nPixels ->
     withGeometry g $ \geometry ->
@@ -188,6 +197,8 @@ spaceTest det pixels rs mlimits doPolarizationCorrection space@(Space fSpace) (D
     withMaybeLimits mlimits rs $ \nlimits limits ->
     withForeignPtr fSpace $ \pSpace -> do
     case img of
+      (ImageDouble arr) -> unsafeWith arr $ \i -> do
+        {-# SCC "test_binoculars_space_test_double" #-} c'hkl_binoculars_space_test_double pSpace geometry sample i nPixels (CDouble . unAttenuation $ att) pix (toEnum ndim) dims r (toEnum nr) mask'' limits (toEnum nlimits) (toEnum . fromEnum $ doPolarizationCorrection)
       (ImageInt32 arr) -> unsafeWith arr $ \i -> do
         {-# SCC "test_binoculars_space_test_int32_t" #-} c'hkl_binoculars_space_test_int32_t pSpace geometry sample i nPixels (CDouble . unAttenuation $ att) pix (toEnum ndim) dims r (toEnum nr) mask'' limits (toEnum nlimits) (toEnum . fromEnum $ doPolarizationCorrection)
       (ImageWord16 arr) -> unsafeWith arr $ \i -> do
@@ -277,19 +288,21 @@ processTestP = do
 
 -- FramesTestP
 
-instance ChunkP (DataFrameTest' DataSourcePath) where
-  chunkP sf sl (DataFrameTest p _) = chunkP sf sl p
+instance ChunkP [DSDataFrameTest DSPath] where
+    chunkP mSkipFirst mSkipLast ps =
+      skipMalformed $ forever $ do
+      sfp <- await
+      withScanFileP sfp $ \f' ->
+        withDataSourcesP f' ps $ \_p p' -> do
+          (DataSourceShape'Range (Z :. f) (Z :. t)) <- ds'Shape p'
+          yield $ cclip (fromMaybe 0 mSkipFirst) (fromMaybe 0 mSkipLast) (Chunk sfp f (t - 1))
 
-instance FramesP (DataFrameTest' DataSourcePath) (DataFrameTest' Identity) where
-  framesP (DataFrameTest qcustom sample) = skipMalformed $ forever $ do
+instance FramesP [DSDataFrameTest DSPath] DataFrameTest where
+  framesP ps = skipMalformed $ forever $ do
     (fp, js) <- await
     withScanFileP fp $ \f ->
-      withDataSourceP f qcustom $ \qcustomAcq ->
-      withDataSourceP f sample $ \sampleAcq ->
-      forM_ js (\j -> tryYield ( DataFrameTest
-                                <$> extract1DStreamValue qcustomAcq j
-                                <*> extract0DStreamValue sampleAcq
-                              ))
+      withDataSourcesP f ps $ \_p p' ->
+      forM_ js (tryYield . extract1DStreamValue p')
 
 ------------
 -- Inputs --
@@ -298,11 +311,12 @@ instance FramesP (DataFrameTest' DataSourcePath) (DataFrameTest' Identity) where
 guess'DataSourcePath'DataFrameTest :: Config Common
                                    -> Config Sample
                                    -> ConfigContent
-                                   -> DataFrameTest' DataSourcePath
+                                   -> DSWrap_ DSDataFrameTest DSPath
 guess'DataSourcePath'DataFrameTest common sample content
-  = DataFrameTest
-    (guess'DataSourcePath'DataFrameQCustom common Nothing content)
-    (guess'DataSourcePath'Sample common sample)
+  = [ DSDataFrameTest
+      (guess'DataSource'DataFrameQCustom common Nothing content)
+      (guess'DataSource'Sample common sample)
+    ]
 
 ---------
 -- Cmd --
