@@ -13,7 +13,7 @@
  * You should have received a copy of the GNU General Public License
  * along with the hkl library.  If not, see <http://www.gnu.org/licenses/>.
  *
- * Copyright (C) 2003-2019, 2023 Synchrotron SOLEIL
+ * Copyright (C) 2003-2019, 2023, 2024, 2025 Synchrotron SOLEIL
  *                         L'Orme des Merisiers Saint-Aubin
  *                         BP 48 91192 GIF-sur-YVETTE CEDEX
  *
@@ -50,6 +50,11 @@
 #define HKL_DEGTORAD (M_PI/180.)
 #define HKL_RADTODEG (180./M_PI)
 
+/**
+ * HKL_VERSION: (type utf8)
+ */
+#define HKL_VERSION "5.1.4"
+
 /* tau = 2pi or 1 */
 #define HKL_TAU (2. * M_PI)
 /* #define HKL_TAU 1 */
@@ -528,7 +533,7 @@ HKLAPI int hkl_engine_pseudo_axis_values_get(HklEngine *self,
 
 HKLAPI HklGeometryList *hkl_engine_pseudo_axis_values_set(HklEngine *self,
 							  double values[], size_t n_values,
-							  HklUnitEnum unit_type, GError **error) HKL_ARG_NONNULL(1, 2) HKL_WARN_UNUSED_RESULT;
+							  HklUnitEnum unit_type, GError **error) HKL_ARG_NONNULL(1) HKL_WARN_UNUSED_RESULT;
 
 HKLAPI const HklParameter *hkl_engine_pseudo_axis_get(const HklEngine *self,
 						      const char *name,
