@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2011 Joseph Adams <joeyadams3.14159@gmail.com>
+ * Copyright (C) 2011, 2025 Joseph Adams <joeyadams3.14159@gmail.com>
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -283,10 +283,11 @@ typedef darray(unsigned long)  darray_ulong;
                 if (NULL == tmp && 0 != (newAlloc) ) {                  \
                         abort();                                        \
                 }else{                                                  \
-                        (arr).item = tmp;                               \
+                        (arr).item = (typeof((arr).item)) tmp;          \
                         (arr).alloc = (newAlloc);                       \
                 }                                                       \
-	} while(0)
+        } while(0)
+
 #define darray_growalloc(arr, need) do { \
 		size_t need_ = (need); \
 		if (need_ > (arr).alloc) \
@@ -353,23 +354,23 @@ Examples:
 
 	darray(int)  arr;
 	int        *i;
-	
+
 	darray_appends(arr, 0,1,2,3,4);
 	darray_appends(arr, -5,-4,-3,-2,-1);
 	darray_foreach(i, arr)
 		printf("%d ", *i);
 	printf("\n");
-	
+
 	darray_free(arr);
-	
+
 
 	typedef struct {int n,d;} Fraction;
 	darray(Fraction) fractions;
 	Fraction        *i;
-	
+
 	darray_appends(fractions, {3,4}, {3,5}, {2,1});
 	darray_foreach(i, fractions)
 		printf("%d/%d\n", i->n, i->d);
-	
+
 	darray_free(fractions);
 */
